import icemac.addressbook.browser.base
import icemac.addressbook.interfaces


class Base(object):
    """Base for search result handlers."""

    @property
    def persons(self):
        addressbook = icemac.addressbook.interfaces.IAddressBook(None)
        session = icemac.addressbook.browser.base.get_session(self.request)
        return [addressbook[id] for id in session['person_ids']]



import icemac.addressbook.testing


class CRUDTests(icemac.addressbook.testing.BrowserTestCase):
    """Testing ..crud.*."""

    def test_new_address_book_can_be_added_and_edited(self):
        # Only managers are allowed to create address books:
        browser = self.get_browser('mgr')
        browser.open('http://localhost/')
        # On the start page there is a link to add an address book:
        browser.getLink('address book').click()
        browser.getControl('title').value = 'test book'
        browser.getControl('Add').click()
        self.assertEqual(['"test book" added.'], browser.get_messages())
        self.assertEqual('http://localhost/AddressBook', browser.url)
        # Editing is done in master data section:
        browser.getLink('Master data').click()
        browser.getLink('Address book').click()
        self.assertEqual(
            'http://localhost/AddressBook/@@edit.html', browser.url)
        # The add form actually stored the values:
        self.assertEqual('test book', browser.getControl('title').value)
        # The edit form is able to change the data:
        browser.getControl('title').value = 'ftest book'
        browser.getControl('Apply').click()
        self.assertEqual(
            ['Data successfully updated.'], browser.get_messages())
        # The edit form submits to itself and shows the stored data:
        self.assertEqual('ftest book', browser.getControl('title').value)

    def test_edit_can_be_canceled(self):
        self.layer['addressbook'].title = u'ftest-ab'
        browser = self.get_browser('mgr')
        browser.open('http://localhost/ab/@@edit.html')
        self.assertEqual('ftest-ab', browser.getControl('title').value)
        browser.getControl('title').value = 'fancy book'
        browser.getControl('Cancel').click()
        self.assertEqual(
            ['No changes were applied.'], browser.get_messages())
        self.assertEqual('ftest-ab', browser.getControl('title').value)

    def test_editor_is_not_allowed_to_delete_all_persons_in_address_book(self):
        from mechanize import HTTPError
        browser = self.get_browser('editor')
        with self.assertRaises(HTTPError) as err:
            browser.open('http://localhost/ab/@@delete_content.html')
        self.assertEqual('HTTP Error 403: Forbidden', str(err.exception))

    def test_visitor_is_not_allowed_to_delete_all_persons_in_address_book(
            self):
        from mechanize import HTTPError
        browser = self.get_browser('visitor')
        with self.assertRaises(HTTPError) as err:
            browser.open('http://localhost/ab/@@delete_content.html')
        self.assertEqual('HTTP Error 403: Forbidden', str(err.exception))

    def test_administrator_is_able_to_delete_all_persons_in_address_book(self):
        # The Administrator is able to delete all persons in the address
        # book on the edit form. Users are not deleted because the are
        # referenced as a safty belt so the current user does not delete his
        # credencials.

        # Make sure there are some persons (with addresses) and some users
        # in the address book:
        from icemac.addressbook.testing import (
            create_person, create_user, create_postal_address)
        ab = self.layer['addressbook']
        create_person(ab, ab, last_name=u'Tester')
        create_person(ab, ab, last_name=u'Tester 2')
        create_user(
            ab, ab, u'Hans', u'User', u'hans@user.de', u'asdf', ['Visitor'])
        create_user(
            ab, ab, u'Kurt', u'Utzr', u'kurt@utzr.ch', u'asdf', ['Editor'])
        t3 = create_person(ab, ab, last_name=u'Tester 3')
        create_postal_address(ab, t3, city=u'Hettstedt')

        # To delete all persons in the address book, the Adminstrator has to
        # open the `edit address book` form and select the `Delete all
        # persons in address book` button there. An `are you sure` form is
        # displayed:
        browser = self.get_browser('mgr')
        browser.open('http://localhost/ab/@@edit.html')
        browser.getControl('Delete all persons in address book').click()
        self.assertEqual(
            'http://localhost/ab/@@delete_content.html', browser.url)

        # When the adminstrator decides not to delete the persons he is led
        # back to the address book's edit form:
        self.assertIn(
            'Do you really want to delete all persons in this address book?',
            browser.contents)
        # The number of persons in the address book is displayed:
        self.assertIn(
            'class="text-widget int-field">5</span>', browser.contents)
        browser.getControl('No, cancel').click()
        self.assertEqual(['Deletion canceled.'], browser.get_messages())
        self.assertEqual('http://localhost/ab/@@edit.html', browser.url)

        # When he decides to delete all persons he is led back to the person
        # list where only the users are still shown:
        browser.getControl('Delete all persons in address book').click()
        browser.getControl('Yes, delete').click()
        self.assertEqual(
            ['Address book contents deleted.'], browser.get_messages())
        self.assertEqual('http://localhost/ab/person-list.html', browser.url)
        self.assertIn('Person-3">User', browser.contents)
        self.assertIn('Person-4">Utzr', browser.contents)


class SecurityTests(icemac.addressbook.testing.BrowserTestCase):
    """Security testing ..crud.*."""

    def test_editor_is_not_allowed_to_edit_address_books_data(self):
        from mechanize import LinkNotFoundError, HTTPError
        # There is no link to edit the address book's data (title) because
        # the editor is not allowed to do so:
        browser = self.get_browser('editor')
        browser.open('http://localhost/ab/@@masterdata.html')
        with self.assertRaises(LinkNotFoundError):
            browser.getLink('Address book')
        # Even opening the URL is not possible:
        with self.assertRaises(HTTPError) as err:
            browser.open('http://localhost/ab/@@edit.html')
        self.assertEqual('HTTP Error 403: Forbidden', str(err.exception))

    def test_visitor_is_not_allowed_to_edit_address_books_data(self):
        from mechanize import LinkNotFoundError, HTTPError
        # There is no link to edit the address book's data (title) because
        # the visitor is not allowed to do so:
        browser = self.get_browser('visitor')
        browser.open('http://localhost/ab/@@masterdata.html')
        with self.assertRaises(LinkNotFoundError):
            browser.getLink('Address book')
        # Even opening the URL is not possible:
        with self.assertRaises(HTTPError) as err:
            browser.open('http://localhost/ab/@@edit.html')
        self.assertEqual('HTTP Error 403: Forbidden', str(err.exception))


class WelcomeTests(icemac.addressbook.testing.BrowserTestCase):
    """Testing ../welcome.pt."""

    def test_the_greeting_after_login_contains_the_address_book_title(self):
        self.layer['addressbook'].title = u'ftest-ab'
        browser = self.get_browser('visitor')
        browser.open('http://localhost/ab')
        self.assertIn(
            '<p>Welcome to ftest-ab. Please select one of the tabs above.</p>',
            browser.contents)



import unittest
import zope.interface
import zope.schema


class IDummy(zope.interface.Interface):

    foo = zope.schema.TextLine(title=u'foo')
    bar = zope.schema.Int(title=u'bar')


class CreateFieldPropertiesTests(unittest.TestCase):
    """Testing ..schema.createFieldProperties()."""

    def test_creates_fieldproperties_on_class(self):
        from ..schema import createFieldProperties
        from zope.schema.fieldproperty import FieldProperty

        class Dummy(object):
            createFieldProperties(IDummy)

        self.assertTrue(isinstance(Dummy.foo, FieldProperty))
        self.assertTrue(isinstance(Dummy.bar, FieldProperty))
        self.assertTrue(Dummy.foo._FieldProperty__field is IDummy['foo'])

    def test_fields_in_omit_are_not_created_on_class(self):
        from ..schema import createFieldProperties
        from zope.schema.fieldproperty import FieldProperty

        class Dummy(object):
            createFieldProperties(IDummy, omit=['foo'])

        self.assertFalse(hasattr(Dummy, 'foo'))
        self.assertTrue(hasattr(Dummy, 'bar'))



import icemac.addressbook.testing
import unittest2 as unittest


class TimeZoneTests(unittest.TestCase):
    """Testing time zone preferences."""

    layer = icemac.addressbook.testing.TEST_BROWSER_LAYER

    def get_browser(
            self, url='http://localhost/ab/++preferences++/ab.timeZone'):
        browser = icemac.addressbook.testing.Browser()
        browser.login('visitor')
        browser.handleErrors = False
        browser.open(url)
        return browser

    def test_default_value_is_UTC(self):
        self.assertEqual(
            ['UTC'], self.get_browser().getControl('Time zone').displayValue)

    def test_changed_value_is_stored(self):
        browser = self.get_browser()
        browser.getControl('Time zone').displayValue = ['Europe/Berlin']
        browser.getControl('Apply').click()
        self.assertIn('Data successfully updated.', browser.contents)
        self.assertEqual(
            ['Europe/Berlin'], browser.getControl('Time zone').displayValue)

    def test_metadata_is_converted_to_selected_time_zone(self):
        from datetime import datetime
        from icemac.addressbook.testing import create_keyword
        from pytz import utc
        from zope.dublincore.interfaces import IZopeDublinCore
        from zope.component import getUtility
        from zope.preference.interfaces import IDefaultPreferenceProvider

        kw = create_keyword(self.layer['addressbook'], u'foo')
        IZopeDublinCore(kw).modified = datetime(2001, 1, 1, tzinfo=utc)
        default_prefs = getUtility(IDefaultPreferenceProvider)
        default_prefs.getDefaultPreferenceGroup('ab.timeZone').time_zone = (
            'Etc/GMT-4')

        browser = self.get_browser(
            'http://localhost/ab/++attribute++keywords/%s' % kw.__name__)
        self.assertIn('01/01/01 04:00', browser.contents)
        self.assertIn('Modification Date (Etc/GMT-4)', browser.contents)
        self.assertIn('Creation Date (Etc/GMT-4)', browser.contents)



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Michael Howitz
# See also LICENSE.txt

import fanstatic
import js.jquery
import js.jqueryui
import os.path


# CSS
css_lib = fanstatic.Library('css', os.path.join('resources', 'css'))


reset_css = fanstatic.Resource(css_lib, 'reset.css')
base_css = fanstatic.Resource(css_lib, 'base.css', depends=[reset_css])
table_css = fanstatic.Resource(css_lib, 'table.css')
form_css = fanstatic.Resource(css_lib, 'form.css')
no_max_content_css = fanstatic.Resource(
    css_lib, 'no_max_content.css', depends=[form_css])
wizard_css = fanstatic.Resource(css_lib, 'wizard.css')


# JavaScript
js_lib = fanstatic.Library('js', os.path.join('resources', 'js'))

masterdata_fields = fanstatic.Resource(
    js_lib, 'masterdata_fields.js', depends=[js.jqueryui.ui_sortable])

table = fanstatic.Resource(
    js_lib, 'table.js', depends=[js.jquery.jquery])



import icemac.addressbook
import icemac.addressbook.browser.resource
import pkg_resources
import zope.contentprovider.provider


class About(object):
    """View class for the about screen."""

    def version(self):
        return pkg_resources.get_distribution('icemac.addressbook').version


class CopyrightContentProvider(
    zope.contentprovider.provider.ContentProviderBase):
    """Content provider for the copyright string."""

    def update(self):
        # Making sure css is rendered in layout.pt:
        icemac.addressbook.browser.resource.base_css.need()

    def render(self):
        return icemac.addressbook.copyright



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2013 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.addressbook
import icemac.addressbook.interfaces
import icemac.addressbook.testing
import unittest2 as unittest
import zope.authentication.interfaces
import zope.catalog.interfaces
import zope.intid.interfaces
import zope.location.interfaces
import zope.pluggableauth.interfaces


class TestAddressbook(unittest.TestCase,
                      icemac.addressbook.testing.InstallationAssertions):

    layer = icemac.addressbook.testing.ZODB_LAYER

    def check_addressbook(self, ab):
        self.assertTrue(zope.location.interfaces.ISite.providedBy(ab))
        self.assertAttribute(
            ab, 'keywords', icemac.addressbook.interfaces.IKeywords)
        self.assertAttribute(
            ab, 'principals',
            zope.pluggableauth.interfaces.IAuthenticatorPlugin,
            name=u'icemac.addressbook.principals')
        self.assertAttribute(
            ab, 'entities',
            icemac.addressbook.interfaces.IEntities)
        self.assertAttribute(
            ab, 'orders',
            icemac.addressbook.interfaces.IOrderStorage)
        self.assertLocalUtility(ab, zope.intid.interfaces.IIntIds)
        self.assertLocalUtility(ab, zope.catalog.interfaces.ICatalog)
        self.assertLocalUtility(
            ab, zope.authentication.interfaces.IAuthentication)

    def setUp(self):
        super(TestAddressbook, self).setUp()
        self.ab = self.layer['addressbook']

    def test_create(self):
        self.check_addressbook(self.ab)

    def test_recall_create_infrastructure(self):
        icemac.addressbook.addressbook.create_address_book_infrastructure(
            self.ab)
        self.check_addressbook(self.ab)

    def test___repr___no___name__(self):
        self.assertEqual("<AddressBook None (None)>",
                         repr(icemac.addressbook.addressbook.AddressBook()))

    def test___repr___no_title(self):
        self.assertEqual("<AddressBook u'ab' (None)>", repr(self.ab))

    def test___repr__(self):
        self.ab.title = u'My address book'
        self.assertEqual("<AddressBook u'ab' (u'My address book')>",
                         repr(self.ab))


class GetAddressBookTests(unittest.TestCase):
    """Testing ..addressbook.get_address_book."""

    layer = icemac.addressbook.testing.ZODB_LAYER

    def test_returns_current_addressbook(self):
        from icemac.addressbook.interfaces import IAddressBook
        self.assertEqual(self.layer['addressbook'], IAddressBook(42))



# Copyright (c) 2008-2013 Michael Howitz
# See also LICENSE.txt
import contextlib
import zope.container.interfaces
import zope.event
import zope.lifecycleevent


@contextlib.contextmanager
def site(site):
    """Context manager to set site in zope.component.hooks."""
    old_site = zope.component.hooks.getSite()
    zope.component.hooks.setSite(site)
    try:
        yield site
    finally:
        zope.component.hooks.setSite(old_site)


def set_site(func):
    "Decorator which does the set-site-dance."
    def decorated(site_obj, *args, **kw):
        with site(site_obj):
            return func(*args, **kw)
    return decorated


def create_obj(class_, *args, **kw):
    """Create an object of class and fire created event."""
    obj = class_(*args)
    zope.event.notify(zope.lifecycleevent.ObjectCreatedEvent(obj))
    for attrib, value in kw.items():
        setattr(obj, attrib, value)
    return obj


create_obj_with_set_site = set_site(create_obj)


def add(parent, obj):
    nc = zope.container.interfaces.INameChooser(parent)
    name = nc.chooseName('', obj)
    parent[name] = obj
    return name


def create_and_add(parent, class_, *args, **kw):
    obj = create_obj(class_, *args, **kw)
    return add(parent, obj)


def iter_by_interface(container, interface):
    "Iterate a container and return only objects providing a specified iface."
    for obj in container.values():
        if interface.providedBy(obj):
            yield obj


def unique_by_attr_factory(attr_name, error_message):
    """Returns a function checking `attr_name` is unique on parent."""
    def unique(obj, event=None):
        """Makes sure `attr_name` is unique on obj's parent.

        May be used as handler for object events.

        """
        if getattr(obj, '__parent__', None) is None:
            return
        container = obj.__parent__
        values = [getattr(x, attr_name) for x in container.values()]
        if len(values) != len(set(values)):
            raise zope.interface.Invalid(error_message)
    return unique



import icemac.addressbook.testing


class AboutTests(icemac.addressbook.testing.BrowserTestCase):
    """Testing ..about."""

    # The about screen displays some information about the application, its
    # version and licenses. It is accessable everywhere in the application.

    def test_about_dialog_is_displayed_on_root_view(self):
        from icemac.addressbook import copyright
        from pkg_resources import get_distribution
        # The root view is basic auth protected, so we have to log-in to see
        # the root view.
        browser = self.get_browser('mgr')
        browser.open('http://localhost')
        # There is a link on the root view pointing to the about dialog:
        browser.getLink(id="about-view").click()
        self.assertEqual('http://localhost/@@about.html',  browser.url)
        # The about dialog displays the version number of the address book:
        self.assertIn(get_distribution('icemac.addressbook').version,
                      browser.contents)
        # ... and the copyright string (it is contained twice, as the footer
        # contains it, too):
        self.assertEqual(2, browser.contents.count(copyright))
        # There is a link to proicons.com as the license for using the icons
        # requires this:
        self.assertEqual(
            'http://www.proicons.com/icon/i141s0/vistaico_toolbar_icons.htm',
            browser.getLink('proicons.com').url)

    def test_about_dialog_is_displayed_on_login_screen(self):
        # The login screen can be accessed by anonymous users:
        browser = self.get_browser()
        browser.open('http://localhost/ab')
        # There is a link on the login view pointing to the about dialog:
        browser.getLink(id="about-view").click()
        self.assertEqual('http://localhost/@@about.html',  browser.url)

    def test_about_dialog_is_displayed_inside_the_application(self):
        # The about screen can be accessed on every object, for example we
        # use a person:
        self.create_person(u'Tester')
        browser = self.get_browser('visitor')
        browser.open('http://localhost/ab/Person')
        browser.getLink(id="about-view").click()
        self.assertEqual('http://localhost/@@about.html',  browser.url)



import icemac.addressbook.testing


class MasterdataSecurityTests(icemac.addressbook.testing.BrowserTestCase):
    """Security testing ..masterdata."""

    # There are some masterdata which can be edited by persons who are
    # allowed to. There is a master data overview which shows all parts the
    # user can see, this differs between the roles.

    def assert_masterdata_links(self, username, link_texts):
        browser = self.get_browser(username)
        browser.open('http://localhost/ab/@@masterdata.html')
        self.assertEqual(
            link_texts, browser.etree.xpath(
                '//ul[@class="bullet"]/li/a/span/child::text()'))

    def test_manager_sees_all_links_in_masterdata(self):
        self.assert_masterdata_links(
            'mgr', ['Address book', 'Keywords', 'Users', 'Entities'])

    def test_editor_sees_some_links_in_masterdata(self):
        self.assert_masterdata_links('editor', ['Keywords', 'Users'])

    def test_visitor_sees_some_links_in_masterdata(self):
        self.assert_masterdata_links('visitor', ['Keywords', 'Users'])



import icemac.addressbook.testing
import icemac.addressbook.browser.testing


class PersonTableSTests(icemac.addressbook.testing.SeleniumTestCase):
    """Selenium testing ..personlist.PersonTable."""

    layer = icemac.addressbook.browser.testing.SELENIUM_SEARCH_LAYER

    def test_checkall_checkbox_deselects_and_reselects_all_persons(self):
        self.login('visitor', 'visitor')
        s = self.selenium
        s.open('/ab/@@multi_keyword.html')
        s.addSelection('id=form-widgets-keywords', 'label=family')
        s.clickAndWait('id=form-buttons-search')
        s.assertValue('name=persons:list', 'on')
        # Deselect
        s.click('css=input.checkall')
        s.assertValue('name=persons:list', 'off')
        # Reselect
        s.click('css=input.checkall')
        s.assertValue('name=persons:list', 'on')



from mock import patch
import icemac.addressbook.testing
import unittest2 as unittest


class NamesTests(unittest.TestCase):
    """Testing .Names."""

    layer = icemac.addressbook.testing.TEST_BROWSER_LAYER

    def setUp(self):
        from icemac.addressbook.testing import create_person
        super(NamesTests, self).setUp()
        ab = self.layer['addressbook']
        self.p1 = create_person(ab, ab, u'Vrozzek', first_name=u'Paul')
        self.p2 = create_person(ab, ab, u'Vranzz', first_name=u'Peter')

    def test_returns_comma_separated_list_of_names_and_count(self):
        from gocept.testing.mock import Property
        from icemac.addressbook.testing import Browser
        browser = Browser()
        browser.login('visitor')
        persons = (
            'icemac.addressbook.browser.search.result.handler.names.Names.'
            'persons')
        with patch(persons, Property()) as persons:
            persons.return_value = [self.p1, self.p2]
            browser.open('http://localhost/ab/@@person-names.html')
            self.assertIn('Paul Vrozzek, Peter Vranzz', browser.contents)
            self.assertIn('Number of persons: 2', browser.contents)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2013 Michael Howitz
# See also LICENSE.txt
# $Id$

import icemac.addressbook.browser.testing
import icemac.addressbook.testing
import zope.testing.renormalizing


def test_suite():
    suite = icemac.addressbook.testing.TestBrowserDocFileSuite(
        "browser/authentication/login.txt",
        "browser/entities/bugfix.txt",
        "browser/entities/delete_choice_value.txt",
        "browser/entities/entities.txt",
        "browser/entities/file.txt",
        "browser/entities/sortorder-fields.txt",
        "browser/entities/sortorder.txt",
        "browser/keyword/keyword.txt",
        "browser/person/clone.txt",
        "browser/person/file.txt",
        "browser/person/person.txt",
        "browser/person/sortorder.txt",
        "browser/person/translation.txt",
        "browser/principals/principals.txt",
        "browser/rootfolder/rootfolder.txt",
        "browser/search/result/handler/export/translation.txt",
        "browser/search/result/handler/export/userfields.txt",
        )
    suite.addTest(
        icemac.addressbook.testing.DocFileSuite(
            "browser/search/result/handler/delete.txt",
            "browser/search/result/handler/export/export.txt",
            "browser/search/search.txt",
            layer=icemac.addressbook.browser.testing.WSGI_SEARCH_LAYER
            ))
    suite.addTest(
        icemac.addressbook.testing.TestBrowserDocFileSuite(
            # Tests which must not run with the default <DATETIME> normalizer:
            "browser/metadata.txt",
            checker=zope.testing.renormalizing.RENormalizing([])))
    return suite



# Copyright (c) 2013 Michael Howitz
# See also LICENSE.txt
from datetime import datetime
from js.jquery_timepicker_addon import timepicker_locales, timepicker
from js.jqueryui import ui_datepicker_locales, base as jqueryui_css
import grokcore.component as grok
import icemac.addressbook.browser.interfaces
import pytz
import z3c.form.browser.text
import z3c.form.converter
import z3c.form.interfaces
import z3c.form.widget
import zope.schema.interfaces
import icemac.addressbook.preferences.utils


class DatetimeDataConverter(z3c.form.converter.DatetimeDataConverter):
    """Converter for z3c.form displaying localized date times.

    Converts stored datetimes to timezone of request.
    Converts datetimes from form to store with time zone of request.

    """
    @property
    def time_zone_name(self):
        """User selected time zone name."""
        return icemac.addressbook.preferences.utils.get_time_zone_name()

    def toWidgetValue(self, value):
        """Convert to time zone user has selected."""
        if value is not self.field.missing_value and value.tzinfo is not None:
            value = value.astimezone(pytz.timezone(self.time_zone_name))
        return super(DatetimeDataConverter, self).toWidgetValue(value)

    def toFieldValue(self, value):
        value = super(DatetimeDataConverter, self).toFieldValue(value)
        if value is None:
            return value
        return datetime(*value.timetuple()[:6],
                        tzinfo=pytz.timezone(self.time_zone_name))


# Map zope.i18n binary pattern to jquery pattern:
# Sources:
# * http://api.jqueryui.com/datepicker
# * http://trentrichardson.com/examples/timepicker/#tp-formatting
# * https://github.com/zopefoundation/zope.i18n/blob/b8fff2676a7575bf2d3248c562f778776e0603ce/src/zope/i18n/interfaces/__init__.py#L377
pattern_mapping = {
    ('y', 2): 'y',
    ('y', 4): 'yy',
    ('M', 1): 'm',
    ('M', 2): 'mm',
    ('M', 3): 'M',
    ('M', 4): 'MM',
    ('D', 1): 'o',
    ('D', 2): 'o',
    ('D', 3): 'oo',
    ('E', 3): 'D',
    ('E', 4): 'DD',
    ('H', 1): 'h',
    ('H', 2): 'HH',
    ('h', 1): 'hh',
    ('S', 1): 'l',
    ('a', 1): 'TT',
    ('a', 2): 'TT',
    }


def zope_i18n_pattern_to_jquery_pattern(pattern):
    """Converts a zope.i18n datetime pattern to one of jquery."""
    # bin_pattern is a list of tuples (char, count) or chars
    bin_pattern = zope.i18n.format.parseDateTimePattern(pattern)
    # pattern_mapping contains only the differences, so by default produce
    # char * count (or the char if it is no tuple):
    pattern = ''.join(
        [pattern_mapping.get(x, x[0] * x[1] if isinstance(x, tuple) else x)
         for x in bin_pattern])
    return pattern


@zope.interface.implementer(
    icemac.addressbook.browser.interfaces.IDatetimeWidget)
class DatetimeWidget(z3c.form.browser.text.TextWidget):
    """Widget to enter date and time using JavaScript picker."""

    klass = 'datetime-widget'

    def update(self):
        super(DatetimeWidget, self).update()
        jqueryui_css.need()
        timepicker.need()

        locale = self.request.locale
        converter = z3c.form.interfaces.IDataConverter(self)
        self.date_pattern = zope_i18n_pattern_to_jquery_pattern(
            locale.dates.getFormatter('date', converter.length).getPattern())
        self.time_pattern = zope_i18n_pattern_to_jquery_pattern(
            locale.dates.getFormatter('time', converter.length).getPattern())

        locale_name = locale.id.language
        if locale_name in timepicker_locales:
            timepicker_locales[locale_name].need()
        if locale_name in ui_datepicker_locales:
            ui_datepicker_locales[locale_name].need()

    def javascript(self):
        return '''
           jQuery('#%(id)s').datetimepicker(
               {dateFormat: '%(date_pattern)s',
                changeMonth: true,
                changeYear: true,
                firstDay: 0,
                minDate: -365,
                maxDate: 700,
                numberOfMonths: 3,
                stepMonths: 3,
                timeFormat: '%(time_pattern)s',
                hourGrid: 4,
                hourMin: 6,
                hourMax: 22,
                minuteGrid: 15,
                stepMinute: 15});
        ''' % self.__dict__


@grok.adapter(
    zope.schema.interfaces.IDatetime,
    icemac.addressbook.browser.interfaces.IAddressBookLayer)
@grok.implementer(z3c.form.interfaces.IFieldWidget)
def DatetimeFieldWidget(field, request):
    """Factory for DatetimeWidget."""
    return z3c.form.widget.FieldWidget(field, DatetimeWidget(request))



import icemac.addressbook.testing


class PrefGroupEditForm(icemac.addressbook.testing.BrowserTestCase):
    """Testing ..browser.PrefGroupEditForm."""

    def test_redirects_to_self_after_cancel(self):
        browser = self.get_browser('visitor')
        url = 'http://localhost/ab/++preferences++/ab.timeZone/@@index.html'
        browser.open(url)
        self.assertEqual(
            ['UTC'], browser.getControl('Time zone').displayValue)
        browser.getControl('Time zone').displayValue = ['Navajo']
        browser.getControl('Cancel').click()
        # After redirect the original value is restored in the field:
        self.assertEqual(
            ['UTC'], browser.getControl('Time zone').displayValue)
        self.assertEqual(url, browser.url)



import icemac.addressbook.testing


class LayoutTests(icemac.addressbook.testing.BrowserTestCase):
    """Testing ../layout.pt."""

    def test_renders_addressbook_title_in_title_tag_and_h1(self):
        self.layer['addressbook'].title = u'My addresses'
        browser = self.get_browser()
        browser.open('http://localhost/ab')
        self.assertEqual('My addresses', browser.title)
        self.assertEqual(
            'My addresses', browser.etree.xpath('//h1/span')[0].text)

    def test_falls_back_to_default_title_if_no_title_set(self):
        browser = self.get_browser()
        browser.open('http://localhost/ab')
        self.assertEqual('icemac.addressbook', browser.title)
        self.assertEqual(
            'icemac.addressbook', browser.etree.xpath('//h1/span')[0].text)

    def test_renders_default_title_on_root_page(self):
        browser = self.get_browser('mgr')  # need to log in to avoid HTTP-401
        browser.open('http://localhost')
        self.assertEqual('icemac.addressbook', browser.title)
        self.assertEqual(
            'icemac.addressbook', browser.etree.xpath('//h1/span')[0].text)



# Copyright (c) 2010-2013 Michael Howitz
# See also LICENSE.txt
import icemac.addressbook.metadata.interfaces
import icemac.addressbook.schema
import persistent
import zope.component
import zope.interface


class EditorMetadataStorage(persistent.Persistent):
    """Storage for editor metadata in annotations."""

    zope.component.adapts(zope.interface.Interface)
    zope.interface.implements(icemac.addressbook.metadata.interfaces.IEditor)
    icemac.addressbook.schema.createFieldProperties(
        icemac.addressbook.metadata.interfaces.IEditor)


editor_metadata_storage = zope.annotation.factory(
    EditorMetadataStorage, key='icemac.metadata')



import icemac.addressbook.testing


class PersonListTests(icemac.addressbook.testing.BrowserTestCase):
    """Testing ..list.PersonList."""

    def test_list_is_initially_empty(self):
        browser = self.get_browser('visitor')
        browser.open('http://localhost/ab')
        browser.getLink('Person list').click()
        self.assertEqual('http://localhost/ab/@@person-list.html', browser.url)
        self.assertIn(
            'There are no persons entered yet, click on "Add person" to '
            'create one.', browser.contents)


class SecurityTests(icemac.addressbook.testing.BrowserTestCase):
    """Security testing ..list.PersonList."""

    def test_visitor_is_not_allowed_to_add_new_persons(self):
        from mechanize import LinkNotFoundError
        browser = self.get_browser('visitor')
        browser.open('http://localhost/ab/@@person-list.html')
        with self.assertRaises(LinkNotFoundError):
            browser.getLink('person')



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Michael Howitz
# See also LICENSE.txt

import fanstatic
import icemac.addressbook.browser.base
import icemac.addressbook.browser.resource
import js.jqueryui
import z3c.preference.browser


lib = fanstatic.Library('prefs', 'resources')
css = fanstatic.Resource(
        lib, 'prefs.css',
        depends=[icemac.addressbook.browser.resource.form_css])
js = fanstatic.Resource(lib, 'prefs.js', depends=[js.jqueryui.effects_fade])


class CategoryEditForm(icemac.addressbook.browser.base.BaseEditForm,
                       z3c.preference.browser.CategoryEditForm):
    """Preference CategoryEditForm which uses CSS of address book."""

    next_url = 'site'
    next_view = 'person-list.html'
    __init__ = z3c.preference.browser.CategoryEditForm.__init__

    def update(self):
        css.need()
        js.need()
        super(CategoryEditForm, self).update()


class PrefGroupEditForm(icemac.addressbook.browser.base.BaseEditForm,
                        z3c.preference.browser.EditForm):
    """Preference group EditForm which uses CSS of address book."""

    __init__ = z3c.preference.browser.EditForm.__init__

    def redirect_to_next_url(self):
        # Stay on form but reload the values from database in case of
        # cancel:
        self.request.response.redirect(self.request.URL)



from mock import Mock, patch
import unittest


class DateTimeColumn_getSortKey_Tests(unittest.TestCase):
    """Testing ..personlist.DateTimeColumn.getSortKey."""

    def callMUT(self, value):
        from ..personlist import DateTimeColumn
        col = DateTimeColumn(Mock(), Mock(), Mock())
        getRawValue = ('icemac.addressbook.browser.personlist.'
                       'DateTimeColumn.getRawValue')
        with patch(getRawValue) as getRawValue:
            getRawValue.return_value = value
            return col.getSortKey(Mock())

    def test_getSortKey_returns_value_which_is_always_compareable(self):
        from datetime import datetime
        from pytz import timezone
        # We use the isoformat as sort key, so comparison does not break if
        # we nix timezone naive and timezone aware datetimes. And yes, we
        # know that this might produce some glitches in the sort order but
        # it is better than an HTTP-500 and better than trying to guess
        # timezone information.
        self.assertEqual('2012-02-02T21:57:00',
                         self.callMUT(datetime(2012, 2, 2, 21, 57)))
        self.assertEqual('2012-02-02T21:57:00+04:00',
                         self.callMUT(datetime(2012, 2, 2, 21, 57,
                                               tzinfo=timezone('Etc/GMT-4'))))
        self.assertEqual('9999-12-31T23:59:59', self.callMUT(None))



import icemac.addressbook.testing


class TestFieldOrder(icemac.addressbook.testing.SeleniumTestCase):

    def test_field_movement(self):
        self.login('mgr')
        sel = self.selenium
        sel.open('/ab/++attribute++entities/icemac.addressbook.person.Person')

        # Keywords-Field has a language depenend title, we store it here for
        # the assertion of the movement later on
        fieldtitle = sel.getText("//tr[4]/td[1]")

        # The Drag'n'drop commands of selenium don't work so we have to
        # emulate them:
        sel.mouseDownAt("//tr[4]/td[1]", "5,5")
        sel.mouseMoveAt("//tr[2]/td[1]", "5,5")
        sel.mouseUpAt("//tr[1]/td[1]", "5,5")
        # The dragged field is stored in the table below the field it was
        # dragged to.
        sel.clickAndWait("entity-fields-save")

        # After saving the position after drag'n'drop was stored.
        self.assertEqual(fieldtitle, sel.getText("//tr[2]/td[1]"))



import icemac.addressbook.testing


class FrontEndTests(icemac.addressbook.testing.SeleniumTestCase):
    """Testing JS funtions of preferences frontend."""

    def test_preferences_front_end(self):
        self.login('mgr')
        sel = self.selenium
        sel.open("/ab/++preferences++/ab")
        # Preference groups are closed by default, fields are not visible:
        sel.assertNotVisible("css=#form-widgets-columns-row")
        # After opening the group the field is shown:
        sel.click("//fieldset[@class='personLists']/legend")
        sel.waitForVisible("css=#form-widgets-columns-row")
        # Clicking on a form element does not close the group:
        sel.addSelection("id=form-widgets-columns-from",
                         "label=person -- birth date")
        sel.click("name=from2toButton")
        sel.assertVisible("css=#form-widgets-columns-row")
        # Clicking on the legend closes the group:
        sel.click("//fieldset[@class='personLists']/legend")
        sel.waitForNotVisible("css=#form-widgets-columns-row")



from mock import patch, Mock
import icemac.addressbook.testing
import unittest2 as unittest


class DatetimeDataConverterTests(unittest.TestCase):
    """Testing ..form.DatetimeDataConverter."""

    def setUp(self):
        from gocept.testing.mock import Property
        super(DatetimeDataConverterTests, self).setUp()
        time_zone_name = ('icemac.addressbook.browser.form.'
                          'DatetimeDataConverter.time_zone_name')
        patcher = patch(time_zone_name, Property())
        time_zone_name = patcher.start()
        time_zone_name.return_value = 'Etc/GMT-4'
        self.addCleanup(patcher.stop)

    def make_one(self):
        from zope.publisher.browser import TestRequest
        from ..form import DatetimeDataConverter
        widget = Mock()
        widget.request = TestRequest()
        field = Mock()
        field.missing_value = None
        return DatetimeDataConverter(field, widget)

    def test_toWidgetValue_renders_datetime_with_tz_localized(self):
        from datetime import datetime
        from pytz import utc
        self.assertEqual(
            u'13/02/01 21:20',
            self.make_one().toWidgetValue(
                datetime(2013, 2, 1, 17, 20, tzinfo=utc)))

    def test_toWidgetValue_renders_native_datetime_unchanged(self):
        from datetime import datetime
        from pytz import utc
        self.assertEqual(
            u'13/02/01 17:20',
            self.make_one().toWidgetValue(datetime(2013, 2, 1, 17, 20)))

    def test_toWidgetValue_renders_missing_value_as_empty_string(self):
        self.assertEqual(u'', self.make_one().toWidgetValue(None))

    def test_toFieldValue_adds_tzinfo(self):
        from datetime import datetime
        from pytz import timezone
        self.assertEqual(
            datetime(2013, 2, 1, 21, 20, tzinfo=timezone('Etc/GMT-4')),
            self.make_one().toFieldValue(u'13/02/01 21:20'))

    def test_toFieldValue_leaves_empty_value_alone(self):
        from datetime import datetime
        from pytz import timezone
        self.assertIsNone(self.make_one().toFieldValue(u''))


class ZopeI18nPatternToJqueryPatternTests(unittest.TestCase):
    """Testing ..form.zope_i18n_pattern_to_jquery_pattern()."""

    def callFUT(self, pattern):
        from ..form import zope_i18n_pattern_to_jquery_pattern
        return zope_i18n_pattern_to_jquery_pattern(pattern)

    def test_converts_German_date_format_correctly(self):
        self.assertEqual('dd.mm.y HH:mm', self.callFUT('dd.MM.yy HH:mm'))

    def test_converts_American_date_format_correctly(self):
        self.assertEqual('m/d/y hh:mm TT', self.callFUT('M/d/yy h:mm a'))


class DatetimeWidgetTests(icemac.addressbook.testing.SeleniumTestCase):
    """Selenium testing ..form.DatetimeWidget."""

    def setUp(self):
        from icemac.addressbook.testing import create_field, create_keyword
        super(DatetimeWidgetTests, self).setUp()
        ab = self.layer['addressbook']
        keyword_entity_name = 'icemac.addressbook.keyword.Keyword'
        create_field(ab, keyword_entity_name, 'Datetime', u'datetime')
        self.kw = create_keyword(ab, u'foobar')

    def test_datetime_widget_renders_javascript_calendar(self):
        self.login('editor', 'editor')
        s = self.selenium
        s.open('/ab/++attribute++keywords/%s' % self.kw.__name__)
        # Activate the datetime field which opens the JavaScript calendar
        s.click('id=form-widgets-Field-1')
        # Click the `now` button:
        s.click("//button[@type='button']")
        # And the `done` button:
        s.click("xpath=(//button[@type='button'])[2]")
        # Save the form:
        s.clickAndWait("id=form-buttons-apply")
        # Successful apply leads back to keyword overview
        s.assertLocation('http://%s/ab/++attribute++keywords' % s.server)



import sys
import zope.schema
import zope.schema.fieldproperty


def createFieldProperties(schema, omit=[]):
    """Creates a FieldProperty fields in `schema` on the class it is called on.

    schema ... interface those fields should be added to class
    omit ... list of field names to be omitted in creation

    """
    frame = sys._getframe(1)
    for name in zope.schema.getFieldNamesInOrder(schema):
        if name in omit:
            continue
        frame.f_locals[name] = zope.schema.fieldproperty.FieldProperty(
            schema[name])



import icemac.addressbook.testing
import unittest


class ResourcesTests(unittest.TestCase):
    """Testing resources."""

    layer = icemac.addressbook.testing.TEST_BROWSER_LAYER

    def test_image_resources_can_be_delivered(self):
        browser = icemac.addressbook.testing.Browser()
        browser.open('http://localhost/++resource++img/favicon.ico')
        self.assertEqual('200 Ok', browser.headers['status'])
